import sys
import subprocess
import gobject
import pygtk
pygtk.require('2.0')
import gtk
import os
from os.path import join, getsize
import dbus
from dbus.mainloop.glib import DBusGMainLoop
import gnomevfs
import gettext
import shutil
import filecmp
import re
import thread
import threading
import time

LOCALEDIR = "/usr/share/parda"
PATH = "/usr/share/parda"
gettext.bindtextdomain('parda', LOCALEDIR)
gettext.textdomain('parda')

import __builtin__
__builtin__._ = gettext.gettext	


def free_space(dev):
	try:
        	stat = os.statvfs(dev)
	except:
        	# This could get called in the event loop as we're shutting down, after
	        # we've unmounted filesystems.
        	return 0
	return stat.f_bsize * stat.f_bavail

def calcular_tamano(dir):
	try:
		total = 0
		for root, dirs, files in os.walk(dir):
                	relpath = root.replace(dir,'')
			print relpath
			if re.search("^\/\.", relpath) != None:
				continue
			tam = 0
			for name in files:
                                if os.path.isfile(join(root, name)):
                                        tam = tam + getsize(join(root, name))
			total = total + tam
		return total
	except OSError, msg:
		print msg
		return -1

def format_size(size):
	"""Format a partition size."""
	# Taken from ubiquity's ubiquity/misc.py
	if size == "":
		size = 0
	if size < 1024:
		unit = 'B'
		factor = 1
	elif size < 1024 * 1024:
		unit = 'kB'
		factor = 1024
	elif size < 1024 * 1024 * 1024:
		unit = 'MB'
		factor = 1024 * 1024
	elif size < 1024 * 1024 * 1024 * 1024:
		unit = 'GB'
		factor = 1024 * 1024 * 1024
	else:
		unit = 'TB'
		factor = 1024 * 1024 * 1024 * 1024
	return '%.1f %s' % (float(size) / factor, unit)

class parda_Backend:
	def __init__(self, frontend):
		self.frontend = frontend
		self.devices = {}
		DBusGMainLoop(set_as_default=True)
		self.bus = dbus.Bus(dbus.Bus.TYPE_SYSTEM)
		hal_obj = self.bus.get_object('org.freedesktop.Hal', '/org/freedesktop/Hal/Manager')
		self.hal = dbus.Interface(hal_obj, 'org.freedesktop.Hal.Manager')

	def detectar_dispositivos(self):
		devices = self.hal.FindDeviceByCapability('storage')
		for device in devices:
			print "\nNUEVO DISPOSITIVO"
			dev_obj = self.bus.get_object('org.freedesktop.Hal', device)
			d = dbus.Interface(dev_obj, 'org.freedesktop.Hal.Device')
			if not self.es_disp_almacenamiento(d):
				continue
			print "ES VALIDO"
			
			# Comprueba si este dispositivo tiene un sistema de ficheros pero no tiene particiones
			if d.GetProperty('block.is_volume'):
				if d.GetProperty('volume.fstype') == 'vfat':
					self.incluir_dispositivo(d)
			else: # En caso constrario, busca todas las particiones del dispositivo
				children = self.hal.FindDeviceStringMatch('info.parent', device)
				c = False
				for child in children:
					dev_obj = self.bus.get_object('org.freedesktop.Hal', child)
					child = dbus.Interface(dev_obj, 'org.freedesktop.Hal.Device')
					if child.GetProperty('block.is_volume'):
						if child.GetProperty('volume.fstype') == 'vfat':
							c = True
							self.incluir_dispositivo(child)

				if not c and d.PropertyExists('storage.removable.media_size'):
					udi = d.GetProperty('info.udi')
					device = str(d.GetProperty('block.device'))
					self.devices[device] = {
					'label' : '',
					'fstype' : '',
					'uuid' : '',
					'mountpoint' : '',
					'udi' : udi,
					'free' : d.GetProperty('storage.removable.media_size'),
					'capacity' : d.GetProperty('storage.removable.media_size'),
					'device' : device
					}
		self.bus.add_signal_receiver(self.dispositivo_detectado, "DeviceAdded", "org.freedesktop.Hal.Manager", "org.freedesktop.Hal", "/org/freedesktop/Hal/Manager")
		self.bus.add_signal_receiver(self.borrar_dispositivo, "DeviceRemoved", "org.freedesktop.Hal.Manager", "org.freedesktop.Hal", "/org/freedesktop/Hal/Manager")
	
	def incluir_dispositivo(self, dev):
		dev_obj = self.bus.get_object("org.freedesktop.Hal", dev.GetProperty('info.parent'))
		device = dbus.Interface(dev_obj, "org.freedesktop.Hal.Device")
		d = device.GetProperty('block.device')
		if self.devices.has_key(d):
			self.devices.pop(d)
			self.frontend.device_removed(d, source=False)

		udi = dev.GetProperty('info.udi')
		mountpoint = str(dev.GetProperty('volume.mount_point'))
		device = str(dev.GetProperty('block.device'))
		self.devices[device] = {
			#'parent' : str(dev.GetProperty('info.parent')).replace('_', ' '),
			'label' : str(dev.GetProperty('volume.label')).replace(' ', '_'),
			'fstype' : str(dev.GetProperty('volume.fstype')),
			'uuid' : str(dev.GetProperty('volume.uuid')),
			'mountpoint' : mountpoint,
			'udi' : str(dev.GetProperty('info.udi')),
			'free' : mountpoint and free_space(mountpoint) or 0,
			'capacity' : dev.GetProperty('volume.size'),
			'device' : device
		}
		self.bus.add_signal_receiver(self.modificacion_estado, 'PropertyModified', 'org.freedesktop.Hal.Device', 'org.freedesktop.Hal', udi, path_keyword='udi')
		print self.devices[device]
		self.frontend.incluir_dispositivo(self.devices[device])

	def borrar_dispositivo(self, udi):
		d = None
		for device in self.devices.itervalues():
			if device['udi'] == udi:
				d = device['device']
				break
		if d:
			self.devices.pop(d)
		self.frontend.borrar_dispositivo(d)

	def dispositivo_detectado(self, udi):
		print udi
		dev_obj = self.bus.get_object("org.freedesktop.Hal", udi)
		dev = dbus.Interface(dev_obj, "org.freedesktop.Hal.Device")
		success = False
		if dev.PropertyExists('block.is_volume') and dev.GetProperty('block.is_volume'):
			if (dev.PropertyExists('storage.bus') and dev.GetProperty('storage.bus') == 'usb') and dev.GetProperty('storage.removable'):
				if dev.GetProperty('volume.fstype') == 'vfat':
					success = True
			else:
				p = dev.GetProperty('info.parent')
				dev_obj = self.bus.get_object('org.freedesktop.Hal', p)
				d = dbus.Interface(dev_obj, 'org.freedesktop.Hal.Device')
				if (d.PropertyExists('storage.bus') and d.GetProperty('storage.bus') == 'usb') and d.GetProperty('storage.removable'):
					if dev.GetProperty('volume.fstype') == 'vfat':
						success = True
			if success:
				self.incluir_dispositivo(dev)
				return
		
		# Look for empty devices.
		if self.IsStorageDevice(dev):
			children = self.hal.FindDeviceStringMatch('info.parent', udi)
			c = False
			for child in children:
				dev_obj = self.bus.get_object('org.freedesktop.Hal', child)
				child = dbus.Interface(dev_obj, 'org.freedesktop.Hal.Device')
				if child.GetProperty('block.is_volume') and child.GetProperty('volume.fstype') == 'vfat':
					c = True
			if not c and dev.PropertyExists('storage.removable.media_size'):
				device = str(dev.GetProperty('block.device'))
				self.devices[device] = {
				'label' : '',
				'fstype' : '',
				'uuid' : '',
				'mountpoint' : '',
				'udi' : udi,
				'free' : dev.GetProperty('storage.removable.media_size'),
				'capacity' : dev.GetProperty('storage.removable.media_size'),
				'device' : device
				}

	def modificacion_estado(self, num_changes, change_list, udi):
		dev_obj = self.bus.get_object("org.freedesktop.Hal", udi)
		dev = dbus.Interface(dev_obj, "org.freedesktop.Hal.Device")
		mountpoint = str(dev.GetProperty('volume.mount_point'))
		device = str(dev.GetProperty('block.device'))
		self.devices[device] = {
		'label' : str(dev.GetProperty('volume.label')).replace(' ', '_'),
		'fstype' : str(dev.GetProperty('volume.fstype')),
		'uuid' : str(dev.GetProperty('volume.uuid')),
		'mountpoint' : mountpoint,
		'udi' : str(dev.GetProperty('info.udi')),
		'free' : mountpoint and free_space(mountpoint) or 0,
		'capacity' : dev.GetProperty('volume.size'),
		'device' : device
		}
		self.frontend.actualizar_listado(self.devices[device])

	def es_disp_almacenamiento(self, d):
		if d.GetProperty('storage.bus') not in ['usb', 'mmc']:
			return False

		drive_types = ['disk',
                       'sd_mmc',
                       'smart_media',
                       'memory_stick',
                       'compact_flash']
		if d.GetProperty('storage.drive_type') in drive_types:
            		return True

class mensaje_Aviso:
        def __init__(self, msg):
#                self.PATH = '/usr/local/jclic/'
                self.cadena = msg

                # Se prepara el mensaje de entrada para su publicacion
                self.indice_linea = self.preparar_mensaje()

                # Ventana del mensaje y contenedor
                self.warning_window = gtk.Window(gtk.WINDOW_TOPLEVEL)
                self.warning_window.set_position(gtk.WIN_POS_CENTER)
                self.warning_window.set_title(_('Parda - Aviso'))
                self.warning_window.set_border_width(10)
                self.warning_window.set_resizable(False)
                self.warning_window.connect("delete_event", self.delete_event)
                self.warning_window.connect("destroy", self.destroy)
                self.vbox = gtk.VBox(False, 30)
                self.panel = gtk.Fixed()
                self.vbox.pack_start(self.panel, True, True, 0)
                self.warning_window.add(self.vbox)


                # Cabecera de la ventana de aviso
                self.imagen_aviso = gtk.Image()
                self.imagen_aviso.set_from_file('%s/img/warning.png' % PATH)
                self.panel.put(self.imagen_aviso, 200, 10)
                self.et_aviso = gtk.Label()
                self.et_aviso.set_markup(_('<b>Mensaje de aviso</b>'))
                self.et_aviso.set_use_markup(True)
                self.panel.put(self.et_aviso, 160, 65)

                # Mensaje adaptado
                self.mensaje = gtk.Label()
                if msg != None:
                        self.mensaje.set_text("%s" % self.cadena)
                self.mensaje.set_justify(gtk.JUSTIFY_CENTER)
                self.panel.put(self.mensaje, 30, 100)


                # Botonera de la ventana
                # El mensaje es dinamico, tamano de la ventana y de la colocacacion de sus elementos
                # depende del numero de lineas del mensaje, indicado por la variable self.indice_linea
                self.boton = gtk.Button(_('Aceptar'))
                self.boton.connect("clicked", self.delete_event, None)
                self.boton.set_size_request(90,30)
                altura_bot = 150 + (12*self.indice_linea)
                self.panel.put(self.boton, 390, altura_bot - 10)


                # Dibujar la ventana de aviso
                altura = 190 + (12*self.indice_linea)
                self.warning_window.set_size_request(500, altura)
                self.imagen_aviso.show()
                self.et_aviso.show()
                self.mensaje.show()
                self.boton.show()
                self.panel.show()
                self.vbox.show()
                self.warning_window.show()

        def delete_event(self, widget, data = None):
                self.warning_window.destroy()
		gtk.main_quit()

        def destroy(self, widget, data = None):
                self.warning_window.destroy()
		gtk.main_quit()

        def preparar_mensaje(self):
                num_car = len(self.cadena)
                act_car = 69  # numero de caracteres admitidos por cada linea
                num_lin = 1
                while act_car <= num_car:
                        while self.cadena[act_car] != ' ': # cada 69 caractreres del mensaje se introduce un salto de linea
                                act_car = act_car - 1
                        self.cadena = self.cadena[:act_car] + '\n' + self.cadena[act_car+1:]
                        num_lin = num_lin + 1
                        act_car = act_car + 70
                return num_lin

class parda_Frontend:
	def __init__(self):
		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.set_position(gtk.WIN_POS_CENTER)
                self.window.set_border_width(10)
		self.window.set_size_request(400, 400)
                self.window.set_title(_('Parda'))
		self.window.set_icon_from_file('%s/img/parda_icon.png' % PATH)

		self.box = gtk.VBox(False, 10)
                self.window.add(self.box)

		self.window.connect("delete_event", self.delete_event)
                self.window.connect("destroy", self.destroy)

		# Lista de dispositivos
		# - Etiqueta de la lista
		self.et_lista = gtk.Label()
                self.et_lista.set_markup(_('<b>Dispositivos Conectados</b>'))
                self.et_lista.set_use_markup(True)
		self.box.pack_start(self.et_lista, False, True, 0)
		# - Lista
		self.contenedor_lista = gtk.ScrolledWindow()
                self.contenedor_lista.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
                self.contenedor_lista.set_shadow_type(gtk.SHADOW_IN)
                self.listado = gtk.ListStore(str, str, str, str)
                self.vista_arbol_lista = gtk.TreeView()
                self.vista_arbol_lista.set_model(self.listado)
		self.contenido_columna_1 = gtk.CellRendererText()
                self.columna_1 = gtk.TreeViewColumn(_('Dispositivo'), self.contenido_columna_1, text=0)
                self.vista_arbol_lista.append_column(self.columna_1)
		self.contenido_columna_2 = gtk.CellRendererText()
                self.columna_2 = gtk.TreeViewColumn(_('Dev'), self.contenido_columna_2, text=1)
		self.columna_2.set_visible(False)
                self.vista_arbol_lista.append_column(self.columna_2)
		self.contenido_columna_3 = gtk.CellRendererText()
                self.columna_3 = gtk.TreeViewColumn(_('Capacidad'), self.contenido_columna_3, text=2)
                self.vista_arbol_lista.append_column(self.columna_3)
		self.contenido_columna_4 = gtk.CellRendererText()
                self.columna_4 = gtk.TreeViewColumn(_('Etiqueta'), self.contenido_columna_4, text=3)
                self.vista_arbol_lista.append_column(self.columna_4)
		self.seleccion_vista = self.vista_arbol_lista.get_selection()
                self.seleccion_vista.set_mode(gtk.SELECTION_SINGLE)
		self.seleccion_vista.connect("changed", self.actualizar_info_dispositivo)
		self.contenedor_lista.add(self.vista_arbol_lista)
                self.contenedor_lista.show_all()
		self.box.pack_start(self.contenedor_lista, True, True, 0)
		# Fin lista de dispositivos
	
		# Cuadro informativo
		# -Cabecera
		self.hbox_info_cab = gtk.HBox(False, 5)
		self.et_cab_info = gtk.Label(_('Dispositivo selecionado: '))
		self.et_cab_info_dev = gtk.Label()
		self.et_cab_info_dev.set_markup(_('<i>dispositivo no seleccionado</i>'))
                self.et_cab_info_dev.set_use_markup(True)
                self.hbox_info_cab.pack_start(self.et_cab_info, False, True, 0)
                self.hbox_info_cab.pack_start(self.et_cab_info_dev, False, True, 0)
		self.et_cab_info.show()
		self.et_cab_info_dev.show()
		self.hbox_info_cab.show()
		self.box.pack_start(self.hbox_info_cab, False, True, 0)
		# -Barra de espacio libre
		self.espacio = gtk.ProgressBar()
		self.box.pack_start(self.espacio, False, True, 0)
		self.espacio.set_text(_('desconocido'))
		self.espacio.show()
		# -Datos del dispositivo
		self.hbox_info_cap = gtk.HBox(False, 5)
		self.et_info_cap = gtk.Label(_('Capacidad: '))
		self.info_cap = gtk.Label()
		self.info_cap.set_markup(_('<i>desconocido</i>'))
                self.info_cap.set_use_markup(True)
                self.hbox_info_cap.pack_start(self.et_info_cap, False, True, 0)
                self.hbox_info_cap.pack_start(self.info_cap, False, True, 0)
		self.et_info_cap.show()
		self.info_cap.show()
		self.hbox_info_cap.show()
		self.box.pack_start(self.hbox_info_cap, False, True, 0)
				
		self.hbox_info_lib = gtk.HBox(False, 5)
		self.et_info_lib = gtk.Label(_('Espacio libre: '))
		self.info_lib = gtk.Label()
		self.info_lib.set_markup(_('<i>desconocido</i>'))
                self.info_lib.set_use_markup(True)
                self.hbox_info_lib.pack_start(self.et_info_lib, False, True, 0)
                self.hbox_info_lib.pack_start(self.info_lib, False, True, 0)
		self.et_info_lib.show()
		self.info_lib.show()
		self.hbox_info_lib.show()
		self.box.pack_start(self.hbox_info_lib, False, True, 0)
				
		self.hbox_info_pm = gtk.HBox(False, 5)
		self.et_info_pm = gtk.Label(_('Punto de montaje: '))
		self.info_pm = gtk.Label()
		self.info_pm.set_markup(_('<i>desconocido</i>'))
                self.info_pm.set_use_markup(True)
                self.hbox_info_pm.pack_start(self.et_info_pm, False, True, 0)
                self.hbox_info_pm.pack_start(self.info_pm, False, True, 0)
		self.et_info_pm.show()
		self.info_pm.show()
		self.hbox_info_pm.show()
		self.box.pack_start(self.hbox_info_pm, False, True, 0)
		# -Boton de copia
		self.hbox_boton_cop = gtk.HBox(False,5)
		self.b_iniciar_cop = gtk.Button()
		self.b_iniciar_cop.set_tooltip_text(_('Realizar copia de seguridad del directorio personal en un dispositivo USB externo'))
		self.imagen_backup = gtk.Image()
		self.imagen_backup.set_from_file('%s/img/backup.png' % PATH)
		self.b_iniciar_cop.set_image(self.imagen_backup)
		#self.b_iniciar_cop.connect("clicked", self.comenzar_copia_seguridad)
		self.b_iniciar_cop.connect("clicked", self.informacion_copia_seguridad)
		self.b_iniciar_cop.set_sensitive(False)
		self.hbox_boton_cop.pack_start(self.b_iniciar_cop, True, False, 0)
		self.b_iniciar_cop.show()
		self.b_restaurar_cop = gtk.Button()
		self.b_restaurar_cop.set_tooltip_text(_('Restaurar copia de seguridad desde un dispositivo USB externo'))
		self.imagen_restore = gtk.Image()
		self.imagen_restore.set_from_file('%s/img/restore.png' % PATH)
		self.b_restaurar_cop.set_image(self.imagen_restore)
		#self.b_restaurar_cop.connect("clicked", self.comenzar_restore_backup)
		self.b_restaurar_cop.connect("clicked", self.informacion_restaurar_copia)
		self.b_restaurar_cop.set_sensitive(False)
		self.hbox_boton_cop.pack_start(self.b_restaurar_cop, True, False, 0)
		self.b_restaurar_cop.show()
		self.hbox_boton_cop.show()
		self.box.pack_start(self.hbox_boton_cop, False, False, 0)
		# Fin de cuadro informativo
	
		self.backend = parda_Backend(self)
		self.backend.detectar_dispositivos()

		self.et_lista.show()
		self.box.show()	
		self.window.show()

	def actualizar_listado(self, disp):
		texto = None
		if disp['udi']:
			drive = self.get_gnome_drive(disp['udi'])
			if drive:
				texto = drive.get_display_name()
			else:
				texto = disp['mountpoint']
		else:
			texto = disp['filename']
		aux = self.vista_arbol_lista.get_model()
		iter = aux.get_iter_first()
		while iter is not None:
			if aux.get_value(iter, 1) == disp['device']:
				aux.set_value(iter, 0, texto)
				aux.set_value(iter, 1, disp['device'])
				aux.set_value(iter, 2, format_size(disp['capacity']))
				aux.set_value(iter, 3, disp['label'])
			iter = aux.iter_next(iter)

	def incluir_dispositivo(self, disp):
		texto = None
		if disp['udi']:
			drive = self.get_gnome_drive(disp['udi'])
			if drive:
				texto = drive.get_display_name()
			else:
				texto = disp['mountpoint']
		else:
			texto = disp['filename']
		self.listado.append([texto, disp['device'], format_size(disp['capacity']), disp['label']])

	def borrar_dispositivo(self, disp):
		borrar = None
		aux = self.vista_arbol_lista.get_model()
		iter = aux.get_iter_first()
		while iter is not None:
			if aux.get_value(iter, 1) == disp:
				borrar = iter
			iter = aux.iter_next(iter)
		if borrar  is not None:
			aux.remove(borrar)
	
	def actualizar_info_dispositivo(self, widget, data = None):
		device = None
		(aux, iter) = self.seleccion_vista.get_selected()
		if iter is not None:
			device = aux.get_value(iter, 1)
		else:
			self.espacio.set_fraction(0.0)
			self.et_cab_info_dev.set_markup(_('<i>dispositivo no seleccionado</i>'))
			self.espacio.set_text(_('desconocido'))
			self.info_cap.set_markup(_('<i>desconocido</i>'))
			self.info_lib.set_markup(_('<i>desconocido</i>'))
			self.info_pm.set_markup(_('<i>desconocido</i>'))
			self.b_iniciar_cop.set_sensitive(False)
			self.b_restaurar_cop.set_sensitive(False)
			
		if device is not None:
			disp = self.backend.devices[device]
			fraccion = (disp['free'] * 100) / disp['capacity']
			self.espacio.set_fraction(1.0 - fraccion/100.0)
			self.espacio.set_text("%d" % fraccion+_('% libre'))
			self.et_cab_info_dev.set_markup(aux.get_value(iter, 0))
			self.info_cap.set_markup(format_size(disp['capacity']))
			self.info_lib.set_markup(format_size(disp['free']))
			self.info_pm.set_markup(disp['mountpoint'])
			self.b_iniciar_cop.set_sensitive(True)
			self.b_restaurar_cop.set_sensitive(True)
			self.dispositivo_seleccionado = disp
	
	def comenzar_copia_seguridad(self, widget, data = None):
		print "Iniciando copia de seguridad"
		self.window_info_cop.destroy()
		print self.borrar_anterior
		self.tam_home = calcular_tamano(os.getenv('HOME'))
		if self.tam_home == -1:
			print "No se puede calcular el tamano"
			aviso = mensaje_Aviso(_('No se puede calcular el tamano'))
		else:
			print format_size(self.tam_home)
			print self.dispositivo_seleccionado['free']
			dircopia = '%s/copia_seguridad_parda/%s' % (self.dispositivo_seleccionado['mountpoint'], os.path.basename(os.environ.get('HOME')))
			tam_dircopia = calcular_tamano(dircopia)
			if  os.access(dircopia, os.F_OK) == False:
				esp_disp = self.dispositivo_seleccionado['free']
				esp_ocup = self.tam_home
			else:
				if self.borrar_anterior == True:
					print "borrado de anterior"
					print tam_dircopia
					esp_disp = self.dispositivo_seleccionado['free'] + tam_dircopia
					esp_ocup = self.tam_home
				else:
					print "actualizado del anterior"
					esp_disp = self.dispositivo_seleccionado['free'] 
					esp_ocup = self.tam_home - tam_dircopia
			print esp_disp
			print esp_ocup
				
			if esp_disp - esp_ocup <= 0:
				self.window.show()
				aviso = mensaje_Aviso(_('El dispositivo seleccionado no dispone de espacion suficiente para realizar la copia de seguridad de su directorio personal'))
			else:
				try:
					self.hilo_copia=thread.start_new_thread(self.copia_completa, (os.getenv('HOME'), '%s/copia_seguridad_parda' % self.dispositivo_seleccionado['mountpoint'], '1'))
				except Exception, e:
					pass
		

	def comenzar_restore_backup(self, widget, data = None):
		print "Iniciando restauracion de copia de seguridad"
		self.window_info_rest.destroy()
		print data
		try:
			print "aqui se debe restaurar"
			self.tam_home = calcular_tamano(data)
			self.hilo_restore = thread.start_new_thread(self.copia_completa, ('%s/copia_seguridad_parda/%s' % (self.dispositivo_seleccionado['mountpoint'], os.path.basename(os.environ.get('HOME'))), os.getenv('HOME'), '2'))
		except Exception, e:
			print e
			pass
	
	def copia_completa(self, dirorigen, dirdestino, tipo):
		self.window.hide()
		print dirorigen
		print dirdestino
		self.progreso_operacion_activar(self.tam_home, tipo)
		self.b_iniciar_cop.set_sensitive(False)
		self.b_restaurar_cop.set_sensitive(False)
		diruser = os.path.basename(dirorigen)
		dirorigen += os.sep
		if tipo == '1':
			if self.borrar_anterior == True:
				self.progreso_operacion_borrado()
				dircopia_anterior = '%s/copia_seguridad_parda/%s' % (self.dispositivo_seleccionado['mountpoint'], os.path.basename(os.environ.get('HOME')))
				shutil.rmtree(dircopia_anterior)
		for dir, subdirs, files in os.walk(dirorigen):

        		if os.path.isabs(dirdestino):
                		relpath = dir.replace(dirorigen,'')
				if re.search("^\.", relpath) != None:
					continue
				if tipo == '2':
		                	backup_dir = os.path.join(dirdestino, relpath)
				else:
					backup_dir = os.path.join(dirdestino, diruser, relpath)
		        else:
	        	        backup_dir = os.path.join(dir, dirdestino)
		        if not os.path.exists(backup_dir):
        		        os.makedirs(backup_dir)
                		print "Creando directorio Destino %s" % backup_dir

		        for f in files:
				if re.search("^\.", f) != None:
					continue

		                filepath = os.path.join(dir, f)
        		        destpath = os.path.join(backup_dir, f)
				if os.path.isfile(filepath):
					self.progreso_operacion_actdatos(filepath, f)
                		try:
                        		if os.path.isfile(destpath) and filecmp.cmp(destpath, filepath, shallow=False):
						time.sleep(0.015)
                                		continue
		                except OSError:
        		                pass
                		try:
                        		print "Copiando %s a %s" % (filepath, destpath)
					shutil.copy2(filepath, destpath)
				except Exception, e:
					print e
        	        	        pass

		self.progreso_operacion_terminar()
		self.b_iniciar_cop.set_sensitive(True)
		self.b_restaurar_cop.set_sensitive(True)

	def progreso_operacion_activar(self, total, tipo):
		if tipo == '1':
                        titulo = _('Realizando copia de seguridad')
                        leyenda = _('Copiando: ')
                        resumen = _('Copia de seguridad completada correctamente')
                elif tipo == '2':
                        titulo = _('Restaurando copia de seguridad')
                        leyenda = _('Copiando: ')
                        resumen = _('Copia de seguridad restaurada correctamente')
                elif tipo == '3':
                        titulo = _('Calculando tamano del directorio personal')
                        leyenda = _('Comprobando')
                        resumen = ''
                elif tipo == '4':
                        titulo = _('Actualizando copia de seguridad')
                        leyenda = _('Actualizando')
                        resumen = _('Copia de seguridad actualizada correctamente')
                else:
                        titulo = _('Operacion desconocida')
                        leyenda = _('Desconocido :')
                        resumen = ''
		
		self.total_progreso=total
		self.acumulado_progreso=0
		
		# creacion de la ventana de progreso	
		self.window_pro = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window_pro.set_position(gtk.WIN_POS_CENTER)
		self.window_pro.set_keep_above(True)
                self.window_pro.set_border_width(10)
		self.window_pro.set_size_request(500, 140)
                self.window_pro.set_title(titulo)
		self.window_pro.set_icon_from_file('%s/img/info.png' % PATH)
		self.window_pro.connect("delete_event", self.delete_event)
                self.window_pro.connect("destroy", self.destroy)
		self.box_pro = gtk.VBox(False, 10)

		self.hbox_info_pro = gtk.HBox(False, 5)
		self.hbox_info_pro.show()
		self.box_pro.pack_start(self.hbox_info_pro, False, True, 5)

		# Informacion proceso de copiado
                self.etiqueta_progreso = gtk.Label(leyenda)
                self.dato_progreso = gtk.Label("")
                self.hbox_info_pro.pack_start(self.etiqueta_progreso, False, True, 0)
                self.hbox_info_pro.pack_start(self.dato_progreso, False, True, 0)
                self.etiqueta_progreso.show()
                self.dato_progreso.show()
		self.progreso = gtk.ProgressBar()
                self.box_pro.pack_start(self.progreso, False, True, 0)
                self.progreso.show()

		# Fin informacion proceso de copiado
		
		self.et_resumen_pro = gtk.Label(resumen)
		self.box_pro.pack_start(self.et_resumen_pro, False, True, 0)
		self.boton_cerrar_pro = gtk.Button(_('Terminar'))
		self.imagen_salir = gtk.Image()
		self.imagen_salir.set_from_file('%s/img/salir.png' % PATH)
		self.boton_cerrar_pro.set_image(self.imagen_salir)
		self.hbox_boton_pro = gtk.HBox(False,5)
		self.hbox_boton_pro.pack_start(self.boton_cerrar_pro, True, False, 0)
		self.hbox_boton_pro.show()
		self.box_pro.pack_start(self.hbox_boton_pro, False, True, 0)
		self.boton_cerrar_pro.connect("clicked", self.destroy)

               	self.window_pro.add(self.box_pro)
		self.window_pro.set_resizable(False)
		self.window_pro.set_deletable(False)
		self.box_pro.show()
		self.window_pro.show()

	def progreso_operacion_terminar(self):
		self.etiqueta_progreso.set_label('')
		self.dato_progreso.set_label('')
        	self.progreso.set_text(_('100% Completado'))
		self.window_pro.set_title(_('Finalizado'))
	        self.progreso.set_fraction(1)
		self.et_resumen_pro.show()
		self.boton_cerrar_pro.show()
		
	def progreso_operacion_borrado(self):
		self.dato_progreso.set_text(_('Borrando copia de seguridad previa'))

	def progreso_operacion_actdatos(self, nombre_abs, nombre_rel):
		time.sleep(0.1)
		self.acumulado_progreso = self.acumulado_progreso + getsize(nombre_abs)
		if self.acumulado_progreso >= self.total_progreso:
        		fraccion = 100
                else:
                       	fraccion = (self.acumulado_progreso * 100) / self.total_progreso
		self.progreso.set_fraction(fraccion/100.0)
        	self.progreso.set_text("%d" % fraccion+_('% Completado'))
                self.dato_progreso.set_label(nombre_rel)

	def informacion_restaurar_copia(self, widget, data = None):
		dircopia = '%s/copia_seguridad_parda/%s' % (self.dispositivo_seleccionado['mountpoint'], os.path.basename(os.environ.get('HOME')))
		nom_user = os.environ.get('USER')
		if  os.access(dircopia, os.F_OK) == False:
			aviso = mensaje_Aviso(_('No se han encontrado copias de seguridad en el dispositivo externo seleccionado para el usuario que utiliza en estos momentos'))
			return 1

		self.window.hide()
		self.window_info_rest = gtk.Window()
		self.window_info_rest.set_position(gtk.WIN_POS_CENTER)
		self.window_info_rest.set_icon_from_file('%s/img/info.png' % PATH)
		self.window_info_rest.set_title(_('Restaurar copia de seguridad'))
		self.window_info_rest.connect("delete_event", self.cancelar_restauracion_copia)
		vbox_info = gtk.VBox(False, 0)
		self.window_info_rest.add(vbox_info)
		hbox_mensaje = gtk.HBox(False, 0)
		vbox_info.pack_start(hbox_mensaje, True, True, 20)
		vbox_info.show()
		hbox_mensaje.show()
		
		mensaje = gtk.Label(_('Se va a proceder a recuperar la copia de seguridad de los datos\ndel usuario "%s" almacenados en el dispositivo externo.\nTenga en cuenta que los datos que sean diferentes\nde su directorio personal van a ser sustituidos por los que\nse encuentran almacenados en la copia de seguridad.') % nom_user)
		mensaje.set_justify(gtk.JUSTIFY_CENTER)
		hbox_mensaje.pack_start(mensaje, False, True, 20)
		mensaje.show()

		hbox_botones = gtk.HBox(True, 5)
		vbox_info.pack_start(hbox_botones, False, True, 10)
		hbox_botones.show()
		boton_cancelar = gtk.Button(_('Cancelar'))
		imagen_cancelar = gtk.Image()
		imagen_cancelar.set_from_file('%s/img/cancelar.png' % PATH)
		boton_cancelar.set_image(imagen_cancelar)
		boton_cancelar.connect('clicked', self.cancelar_restauracion_copia)
		boton_siguiente = gtk.Button(_('Siguiente'))
		imagen_siguiente = gtk.Image()
		imagen_siguiente.set_from_file('%s/img/siguiente.png' % PATH)
		boton_siguiente.set_image(imagen_siguiente)
		boton_siguiente.connect('clicked', self.comenzar_restore_backup, dircopia)
		hbox_botones.pack_start(boton_cancelar, True, False, 10)
		hbox_botones.pack_start(boton_siguiente, True, False, 10)
		boton_cancelar.show()
		boton_siguiente.show()
		self.window_info_rest.show()

	def informacion_copia_seguridad(self, widget, data = None):
		dircopia = '%s/copia_seguridad_parda/%s' % (self.dispositivo_seleccionado['mountpoint'], os.path.basename(os.environ.get('HOME')))
		nom_user = os.environ.get('USER')
		if  os.access(dircopia, os.F_OK) == False:
			# Esto no funciona, hay que hacer algo para cuando no esta creado el directorio de copia en el destino
			print "Hacer directamente la copia de seguridad"
			mensaje = gtk.Label(_('En el dispositivo seleccionado no existe una copia de seguridad previa del usuario "%s".\nSe va ha realizar una copia de seguridad de sus datos.') % nom_user)
		else:
			mensaje = gtk.Label(_('Se ha encontrado en el dispositivo externo seleccionado una copia de seguridad previa del usuario "%s".\nSeleccione una de las opciones siguientes:') % nom_user)
		
		self.borrar_anterior = False
		self.window.hide()
		self.window_info_cop = gtk.Window()
		self.window_info_cop.set_position(gtk.WIN_POS_CENTER)
		self.window_info_cop.set_icon_from_file('%s/img/info.png' % PATH)
		self.window_info_cop.set_title(_('Crear copia de seguridad'))
		self.window_info_cop.connect("delete_event", self.cancelar_copia_seguridad)
		vbox_info = gtk.VBox(False, 0)
		self.window_info_cop.add(vbox_info)
		hbox_mensaje = gtk.HBox(False, 0)
		vbox_info.pack_start(hbox_mensaje, True, True, 20)
		vbox_info.show()
		hbox_mensaje.show()
		
		#mensaje.set_justify(gtk.JUSTIFY_CENTER)
		hbox_mensaje.pack_start(mensaje, False, True, 20)
		mensaje.show()

		if  os.access(dircopia, os.F_OK) != False:
			hbox_rad_boton = gtk.HBox(False, 0)
			vbox_rad_boton = gtk.VBox(False, 5)
			vbox_info.pack_start(hbox_rad_boton, True, True, 0)
			hbox_rad_boton.pack_start(vbox_rad_boton, True, True, 20)
			vbox_rad_boton.show()
			hbox_rad_boton.show()

			rad_boton = gtk.RadioButton(None, _('Borrar la copia de seguridad previa y crearla de nuevo\n(Recomendado - tiempo de copia mayor)'))
			rad_boton.connect("toggled", self.seleccion_borrar, True)
			vbox_rad_boton.pack_start(rad_boton, True, True, 0)
			rad_boton.show()

			rad_boton = gtk.RadioButton(rad_boton, _('Actualizar la copia de seguridad con los datos actualizados de su directorio personal\n(Menor tiempo de copia, no se borran ficheros obsoletos)'))
			rad_boton.connect("toggled", self.seleccion_borrar, False)
			rad_boton.set_active(True)
			vbox_rad_boton.pack_start(rad_boton, True, True, 0)
			rad_boton.show()

		hbox_botones = gtk.HBox(True, 5)
		vbox_info.pack_start(hbox_botones, False, True, 10)
		hbox_botones.show()
		boton_cancelar = gtk.Button(_('Cancelar'))
		imagen_cancelar = gtk.Image()
		imagen_cancelar.set_from_file('%s/img/cancelar.png' % PATH)
		boton_cancelar.set_image(imagen_cancelar)
		boton_cancelar.connect('clicked', self.cancelar_copia_seguridad)
		boton_siguiente = gtk.Button(_('Siguiente'))
		imagen_siguiente = gtk.Image()
		imagen_siguiente.set_from_file('%s/img/siguiente.png' % PATH)
		boton_siguiente.set_image(imagen_siguiente)
		boton_siguiente.connect('clicked', self.comenzar_copia_seguridad, None)
		hbox_botones.pack_start(boton_cancelar, True, False, 10)
		hbox_botones.pack_start(boton_siguiente, True, False, 10)
		boton_cancelar.show()
		boton_siguiente.show()
		self.window_info_cop.show()

	def seleccion_borrar(self, widget, data = None):
		if widget.get_active():
			self.borrar_anterior = data

	def cancelar_copia_seguridad(self, widget, data = None):
		self.window_info_cop.destroy()
		self.window.show()

	def cancelar_restauracion_copia(self, widget, data = None):
		self.window_info_rest.destroy()
		self.window.show()

	def get_gnome_drive(self, udi):
		monitor = gnomevfs.VolumeMonitor()
		for drive in monitor.get_connected_drives():
			if drive.get_hal_udi() == udi:
				return drive

	def delete_event(self, widget, data = None):
		gtk.main_quit()

	def destroy(self, widget, data = None):
		gtk.main_quit()

def main():
	gtk.gdk.threads_init()
        gtk.main()

if __name__ =="__main__":
	front = parda_Frontend()
	main()
